//+------------------------------------------------------------------+
//|                                                 IncADOnArray.mqh |
//|                          https://login.mql5.com/en/users/Integer |
//+------------------------------------------------------------------+
#property copyright "Integer"
#property link      "https://login.mql5.com/en/users/Integer"
#property version   "1.00"
/*
   External parameters:
   // Indicator period: positive value or 0 - all bars.
   input int      ADPeriod=0; 

   Declaration:
   #include <IncOnArray/IncADOnArray.mqh>
   CADOnArray ad;

   In OnInit:  
   ad.Init(ADPeriod);   
   
   or 
   ad.Init();

   In OnCalculate: 
   ad.Solve(rates_total,prev_calculated,DataHigh,DataLow,DataClose,DataVolume,ADBuffer); 
*/
//+------------------------------------------------------------------+
//| CADOnArray                                                       |
//+------------------------------------------------------------------+
class CADOnArray
  {
protected:
   int               m_Period;
   string            m_Name;
   int               m_br;
public:
   void Init(int aPeriod=0)
     {
      m_Period=aPeriod;
      m_br=0;
      m_Name="AD";
      if(m_Period>0)
        {
         m_br=m_Period;
         m_Name="AD("+IntegerToString(m_Period)+")";
        }
     }
   void Solve(const int aRatesTotal,
              const int aPrevCalc,
              double  &aDataHigh[],
              double  &aDataLow[],
              double  &aDataClose[],
              double  &aDataVolume[],
              double  &aAD[])
     {
      int Start=0;
      if(aPrevCalc==0)
        {
         for(int i=0;i<aRatesTotal;i++)
           {
            if(aDataHigh[i]!=0 && aDataHigh[i]!=EMPTY_VALUE)
              {
               if(aDataLow[i]!=0 && aDataLow[i]!=EMPTY_VALUE)
                 {
                  if(aDataClose[i]!=0 && aDataClose[i]!=EMPTY_VALUE)
                    {
                     if(aDataVolume[i]!=0 && aDataVolume[i]!=EMPTY_VALUE)
                       {
                        Start=i;
                        break;
                       }
                    }
                 }
              }
           }
         if(m_Period>0)
           {
            Start=Start+m_Period-1;
            aAD[Start]=0;
            for(int i=Start;i>Start-m_Period;i--)
              {
               double m_sum=0;
               if(aDataHigh[i]!=aDataLow[i])
                 {
                  m_sum=(aDataClose[i]-aDataLow[i])-(aDataHigh[i]-aDataClose[i]);
                  m_sum=(m_sum/(aDataHigh[i]-aDataLow[i]))*aDataVolume[i];
                 }
               aAD[Start]+=m_sum;
              }
           }
         else
           {
            int i=Start;
            aAD[i]=0;
            if(aDataHigh[i]!=aDataLow[i])
              {
               aAD[i]=(aDataClose[i]-aDataLow[i])-(aDataHigh[i]-aDataClose[i]);
               aAD[i]=(aAD[i]/(aDataHigh[i]-aDataLow[i]))*aDataVolume[i];
              }
           }
         for(int i=0;i<Start;i++)
           {
            aAD[i]=EMPTY_VALUE;
           }
         Start++;
        }
      else
        {
         Start=aPrevCalc-1;
        }
      if(m_Period>0)
        {
         for(int i=Start;i<aRatesTotal;i++)
           {
            double m_sum=0;
            if(aDataHigh[i]!=aDataLow[i])
              {
               m_sum=(aDataClose[i]-aDataLow[i])-(aDataHigh[i]-aDataClose[i]);
               m_sum=(m_sum/(aDataHigh[i]-aDataLow[i]))*aDataVolume[i];
              }
            double m_sum_o=0;
            int j=i-m_Period;
            if(aDataHigh[j]!=aDataLow[j])
              {
               m_sum_o=(aDataClose[j]-aDataLow[j])-(aDataHigh[j]-aDataClose[j]);
               m_sum_o=(m_sum_o/(aDataHigh[j]-aDataLow[j]))*aDataVolume[j];
              }
            aAD[i]=aAD[i-1]-m_sum_o+m_sum;
           }
        }
      else
        {
         for(int i=Start;i<aRatesTotal;i++)
           {
            double m_sum=0;
            if(aDataHigh[i]!=aDataLow[i])
              {
               m_sum=(aDataClose[i]-aDataLow[i])-(aDataHigh[i]-aDataClose[i]);
               m_sum=(m_sum/(aDataHigh[i]-aDataLow[i]))*aDataVolume[i];
              }
            aAD[i]=aAD[i-1]+m_sum;
           }
        }
     }
   int BarsRequired()
     {
      return(m_br);
     }
   string Name()
     {
      return(m_Name);
     }
   string About()
     {
      return("Integer's ADOnArray class. https://login.mql5.com/en/users/Integer");
     }
  };
//+------------------------------------------------------------------+
